%% Local Activity Distribution Analysis

%This program generates the pixel activity distribution as a function of
%time. It uses the subtracted reflected microscopy images as input to show
%how the surface changes of the intermetallic particles being analysed
%progresses with time. Extraction of the activity specific to each
%intermetallic particle is possible through the ROI set established from
%pre-immersion SEM-EDX analysis of the sample surface.

%Main inputs
    %time series subtracted images (unthresholded)
    %time series subtracted images (with threshold applied)
    %unzipped folder containing ROIs of the intermetallic particles being
    %analysed
    
%Main outputs
    %numerical data corresponding to the pixel activity level distribution
    %of the intermetallic particles being analysed
    %plots visualising the pixel activity level distributio as a function
    %of time
    %the numerical data and plots were generated for the entire RROI, and
    %the IMP and adjacent matrix subregions.

%% import inputs

clc; clear;

%Import processed image set (i.e. ImageJ outputs compared wrt t=0)
dir1=uigetdir('C:\','Select folder containing INPUT 8-bit subtracted images');
if dir1==0
    errordlg('Program Terminated. No input folder specified.');
    return
end

%Input parameters used for d2 calculation
prompt={'Raw Image Frame Rate (fps)','Correlation Range (s)','Averaging Radius (s)','Step Size (Frames)','File Type','ROI Margin (pix)','End Time(s)','Lower Th','t supply'};
dlgtitle='Input kinetic calculation parameters';
dims=[1 50];
definput={'1','0','0','1','tif','4','600','5','0'};
answer=inputdlg(prompt,dlgtitle,dims,definput);

if isempty(answer)
    errordlg('Program terminated. No calculation parameters specified.');
    return
else
    FR=str2double(answer{1});
    CR=str2double(answer{2});
    AW=str2double(answer{3});
    SS=str2double(answer{4});
    FT=char(answer(5)); lft=length(FT);
    w=str2double(answer(6));
    dw=str2double(answer(7));
    lth=str2double(answer(8));
    tsupply=str2double(answer(9));
end

%Output folder
outpath=uigetdir('C:\','Select folder for OUTPUT file');
if outpath==0
    errordlg('Program Terminated. No output folder specified.');
    return
end

%Import ROIs
dir2=uigetdir('C:\','Select folder containing corresponding ROI files');
if dir2==0
    errordlg('Program Terminated. No input folder specified.');
    return
end

%Image directory
wb=waitbar(0,"Initiating Procedure");
tic;

listI=dir(fullfile(dir1,['*' FT]));
LI=length(listI);

%ROI directory
listR=dir(fullfile(dir2,'*roi'));
LR=length(listR);

%Output Data
dirn=['P2_SegLocKntcDataInt' '_Margin=' num2str(w,'%03.0f') 'px_' 'loclth=' num2str(lth,'%03.0f')];
f0=fullfile(outpath,dirn);
mkdir(f0)

dirn_num='P2.1_LocKntcInt_Num';
f1=fullfile(outpath,dirn,dirn_num);
mkdir(f1)

dirn_plt='P2.2_LocKntcInt_HistoEvol';
f2=fullfile(outpath,dirn,dirn_plt);
mkdir(f2)

dirn_hist='P2.3_LocKntcInt_PixelCount';
f3=fullfile(outpath,dirn,dirn_hist);
mkdir(f3)

dirn_num_IM='P2.4_LocKntcInt_Num_IM';
f4=fullfile(outpath,dirn,dirn_num_IM);
mkdir(f4)

dirn_plt_IM='P2.5_LocKntcInt_HistoEvol_IM';
f5=fullfile(outpath,dirn,dirn_plt_IM);
mkdir(f5)

dirn_hist_IM='P2.6_LocKntcInt_PixelCount_IM';
f6=fullfile(outpath,dirn,dirn_hist_IM);
mkdir(f6)

dirn_num_MT='P2.7_LocKntcInt_Num_MX';
f7=fullfile(outpath,dirn,dirn_num_MT);
mkdir(f7)

dirn_plt_MT='P2.8_LocKntcInt_HistoEvol_MX';
f8=fullfile(outpath,dirn,dirn_plt_MT);
mkdir(f8)

dirn_hist_MT='P2.9_LocKntcInt_PixelCount_MX';
f9=fullfile(outpath,dirn,dirn_hist_MT);
mkdir(f9)

%Pixel Analysis
waitbar(0,wb,sprintf('Analyzing ROI Pixels\n (ETA: calculating...)'));
lwbt=length(3:LR);
lwb=0;

for i=1:LR
    r=listR(i);
    rn=[r.folder '\' r.name];
    roi=ROIread(rn);
    
    crc = roi.vnRectBounds+1; %[t l b r]
    IMc = roi.nStrokeColor;
    
    pCol=argb2rgb(IMc);
    
    if sum(pCol~=[1,1,0]')>0
        %Setup matrix for pixel dynamic values
        kin=zeros(length(1:SS:dw),263);
        kinIM=zeros(length(1:SS:dw),263);
        kinMT=zeros(length(1:SS:dw),263);
        
        %Analysis boundary
        ab=[crc(1)-w crc(2)-w crc(3)+w crc(4)+w]; %[t l b r] analysis margins
        
        %Extract pixel data from 8-bit dynamic images
        ct=0;
        
        for j=1:SS:LI
            di=listI(j);
            imn=[di.folder '\' di.name];
            imo=(imread(imn)); %least active is 0, most active is 255 (i.e. dark barkground)
            
            %mask generation
            if j==1
                pxc0=pxconnectf(roi.mnCoordinates); %[col row]
                pxc=[pxc0(:,2) pxc0(:,1)]; %[row col]
                immask=ROImask(imo,pxc);
            end
            
            
            %original image
            %crop image
            imc=imo(ab(1):ab(3),ab(2):ab(4));
            
            %get histogram
            [intvals0,intv]=imhist(imc);
            
            %median search
            frqdist=intvals0./sum(intvals0);
            
            %affected area curves
            %local thresholding
            ath=lth/255;
            imb=imbinarize(imc,ath); %thresholds and creates mask for the analysis margin
            
            %despeckle image
            imbf=medfilt2(imb,[5 5],'symmetric'); %radius = 2
            imbfh=imhist(imbf); %counts the pixels that are above and below threshold
            
            %store values
            ct=ct+1;
            kin(ct,1)=ct; %index number
            kin(ct,2:257)=intvals0; %intensity distribution of unthresholded subtract image
            kin(ct,258)=find(intvals0,1,'first')-1; %lowest non-zero intensity
            kin(ct,259)=find(intvals0,1,'last')-1; %largest non-zero intensity
            kin(ct,260)=lth; %fixed threshold
            kin(ct,261)=imbfh(2)/sum(imbfh); %active pixel fraction
            kin(ct,263)=midGLsearch(frqdist);
            
            %timestamp
            ln=length(di.name);
            tst=di.name((ln-(lft+1+9-1)):(ln-(lft+1))); %DD_HHMMSS
            if ct==1; tstart=tst; end %data in string form
            
            %elapsed time calculations
            ehr=(str2double(tst(1:2))-str2double(tstart(1:2)))*24+str2double(tst(4:5))-str2double(tstart(4:5));
            emn=(str2double(tst(6:7))-str2double(tstart(6:7)));
            esc=(str2double(tst(8:9))-str2double(tstart(8:9)));
            kin(ct,262)=ehr*3600+emn*60+esc;
            
            
            %SEGMENTED Image IM
            maskct=immask(ab(1):ab(3),ab(2):ab(4)); %%
            sizerc=size(maskct);
            matct=length(find(maskct>0));
            matpix=(sizerc(1)*sizerc(2))-matct;
            
            maskedim=imo.*immask;
            imcmask=maskedim(ab(1):ab(3),ab(2):ab(4));
            [intvalsIM,intvIM]=imhist(imcmask);
            intvalsIM(1)=intvalsIM(1)-matpix;
            
            %median search
            frqdistIM=intvalsIM./sum(intvalsIM);
            kinIM(ct,263)=midGLsearch(frqdistIM);
            
            %affected area curves
            %local thresholding
            imbIM=imbinarize(imcmask,ath); %thresholds and creates mask for the analysis margin
            
            %despeckle image
            imbfIM=medfilt2(imbIM,[5 5],'symmetric'); %radius = 2
            imbfhIM=imhist(imbfIM); %counts the pixels that are above and below threshold
            
            %store values
            kinIM(ct,1)=ct; %index number
            kinIM(ct,2:257)=intvalsIM; %intensity distribution of unthresholded subtract image
            kinIM(ct,258)=find(intvalsIM,1,'first')-1; %lowest non-zero intensity
            kinIM(ct,259)=find(intvalsIM,1,'last')-1; %largest non-zero intensity
            kinIM(ct,260)=lth; %fixed threshold
            kinIM(ct,261)=imbfhIM(2)/sum(imbfh); %active pixel fraction relative to RROI
            kinIM(ct,262)=kin(ct,262);
            
            %SEGMENTED Image matrix
            %store values
            intvalsMT=intvals0-intvalsIM;
            
            %median search
            frqdistMT=intvalsMT./sum(intvalsMT);
            kinMT(ct,263)=midGLsearch(frqdistMT);
            
            kinMT(ct,1)=ct; %index number
            kinMT(ct,2:257)=intvalsMT; %intensity distribution of unthresholded subtract image
            kinMT(ct,258)=find(intvalsMT,1,'first')-1; %lowest non-zero intensity
            kinMT(ct,259)=find(intvalsMT,1,'last')-1; %largest non-zero intensity
            kinMT(ct,260)=lth; %fixed threshold
            kinMT(ct,261)=(imbfh(2)-imbfhIM(2))/sum(imbfh); %active pixel fraction
            kinMT(ct,262)=kin(ct,262);
            
            if kin(ct,262)>=dw; break; end
        end
        
        %delete excess
        kin(ct+1:end,:)=[];
        kinIM(ct+1:end,:)=[];
        kinMT(ct+1:end,:)=[];
        
        %save numerical data
        nn=[r.name(1:(length(r.name)-4)) '.csv'];
        sn1=fullfile(f1,nn);
        writematrix(kin,sn1);
        
        nnIM=[r.name(1:(length(r.name)-4)) '.csv'];
        sn1IM=fullfile(f4,nnIM);
        writematrix(kinIM,sn1IM);
        
        nnMT=[r.name(1:(length(r.name)-4)) '.csv'];
        sn1MT=fullfile(f7,nnMT);
        writematrix(kinMT,sn1MT);
        
        %save histogram evolution plot image_OG Plot
        fg=figure('visible','off');
        yy=0:255;
        xx=kin(:,262);
        [XX,YY]=meshgrid(xx,yy);
        ps=surf(XX,YY,kin(:,2:257)'./sum(imbfh)*100);
        view(0,90);
        
        xlim([0 dw]);
        xticks(0:dw/6:dw);
        ylim([0 255]);
        yticks(0:25:255)
        xlabel('Time (s)')
        ylabel('Gray-Level')
        zlabel('S change (%)')
        
        %colormap specs
        nm2=cool; nm2(1:256,:)=nm2(256:-1:1,:,:);
        nm1=spring; nm1(1:256,:)=nm1(256:-1:1,:,:);
        tp=0.05;
        tt=floor(tp*255);
        v1=linspace(1,256,tt); k1=floor(v1);
        nm(1:tt,:)=nm1(k1,:);
        v2=linspace(1,256,256-tt); k2=floor(v2);
        nm(tt+1:256,:,:)=nm2(k2,:);
        nm(1,:)=[0.9 0.9 0.9];
        
        colormap(nm)
        
        colorbar
        ps.EdgeColor='none';
        
        pn=[r.name(1:(length(r.name)-4)) '.jpg'];
        sn2=fullfile(f2,pn);
        exportgraphics(fg,sn2,'Resolution',600);
        
        %save histogram evolution plot image_IM Plot
        fgIM=figure('visible','off');
        yy=0:255;
        xx=kin(:,262);
        [XX,YY]=meshgrid(xx,yy);
        ps=surf(XX,YY,kinIM(:,2:257)'./sum(imbfh)*100);
        view(0,90);
        
        xlim([0 dw]);
        xticks(0:dw/6:dw);
        ylim([0 255]);
        yticks(0:25:255)
        xlabel('Time (s)')
        ylabel('Gray Level')
        zlabel('S change (%)')
        
        %colormap specs
        nm2=cool; nm2(1:256,:)=nm2(256:-1:1,:,:);
        nm1=spring; nm1(1:256,:)=nm1(256:-1:1,:,:);
        tp=0.05;
        tt=floor(tp*255);
        v1=linspace(1,256,tt); k1=floor(v1);
        nm(1:tt,:)=nm1(k1,:);
        v2=linspace(1,256,256-tt); k2=floor(v2);
        nm(tt+1:256,:,:)=nm2(k2,:);
        nm(1,:)=[0.9 0.9 0.9];
        
        colormap(nm)
        
        colorbar
        ps.EdgeColor='none';
        
        pnIM=[r.name(1:(length(r.name)-4)) '.jpg'];
        sn2IM=fullfile(f5,pnIM);
        exportgraphics(fgIM,sn2IM,'Resolution',600);
        
        %save histogram evolution plot image_MT Plot
        fgMT=figure('visible','off');
        yy=0:255;
        xx=kinMT(:,262);
        [XX,YY]=meshgrid(xx,yy);
        ps=surf(XX,YY,kinMT(:,2:257)'./sum(imbfh)*100);
        view(0,90);
        
        xlim([0 dw]);
        xticks(0:dw/6:dw);
        ylim([0 255]);
        yticks(0:25:255)
        xlabel('Time (s)')
        ylabel('Gray-level')
        zlabel('S change (%)')
        
        %colormap specs
        nm2=cool; nm2(1:256,:)=nm2(256:-1:1,:,:);
        nm1=spring; nm1(1:256,:)=nm1(256:-1:1,:,:);
        tp=0.05;
        tt=floor(tp*255);
        v1=linspace(1,256,tt); k1=floor(v1);
        nm(1:tt,:)=nm1(k1,:);
        v2=linspace(1,256,256-tt); k2=floor(v2);
        nm(tt+1:256,:,:)=nm2(k2,:);
        nm(1,:)=[0.9 0.9 0.9];
        
        colormap(nm)
        
        colorbar
        ps.EdgeColor='none';
        
        pnMT=[r.name(1:(length(r.name)-4)) '.jpg'];
        sn2MT=fullfile(f8,pnMT);
        exportgraphics(fgMT,sn2MT,'Resolution',600);
        
        %save histogram span and pixel count plot image_OG
        fg2=figure('visible','off');
        
        la=plot(kin(:,262),kin(:,260));
        hold on
        lb=plot(kin(:,262),kin(:,259));
        lc=plot(kin(:,262),kin(:,258));
        ld=plot(kin(:,262),kin(:,263));
        ylim([0 255]);
        yticks(0:25:255)
        ylabel('Gray-Level')
        hold off
        
        yyaxis right
        le=plot(kin(:,262),kin(:,261)*100);
        hold on
        lf=plot(ones(1,100)*tsupply,linspace(0,100,100),'--');
        hold on
        ylabel('S change(%)')
        ylim([0 100])
        yticks(0:10:100)
        
        legend([la,lb,lc,ld,le,lf],'lth','Max','Min','Mid','Sc%','ts')
        legend('Location','northoutside','Orientation','horizontal')
        
        xlabel('Time (s)')
        xlim([0 dw]);
        xticks(0:dw/6:dw)
        
        pn2=[r.name(1:(length(r.name)-4)) '.jpg'];
        sn3=fullfile(f3,pn2);
        exportgraphics(fg2,sn3,'Resolution',600);
        
        %SEGMENTED_IM
        %save histogram span and pixel count plot image_SEGMENTED IM
        fg2IM=figure('visible','off');
        
        yyaxis left
        laIM=plot(kinIM(:,262),kinIM(:,260));
        hold on
        lbIM=plot(kinIM(:,262),kinIM(:,259));
        lcIM=plot(kinIM(:,262),kinIM(:,258));
        ldIM=plot(kinIM(:,262),kinIM(:,263));
        ylim([0 255]);
        yticks(0:25:255)
        ylabel('Pixel Gray-Level')
        hold off
        
        yyaxis right
        leIM=plot(kinIM(:,262),kinIM(:,261)*100);
        hold on
        lfIM=plot(ones(1,100)*tsupply,linspace(0,100,100),'--');
        hold off
        ylabel('S change (%)')
        ylim([0 100])
        yticks(0:10:100)
        
        legend([laIM,lbIM,lcIM,ldIM,leIM,lfIM],'lth','Max','Min','Mid','Sc%','ts')
        legend('Location','northoutside','Orientation','horizontal')
        
        xlabel('Time (s)')
        xlim([0 dw]);
        xticks(0:dw/6:dw)
        
        pn2IM=[r.name(1:(length(r.name)-4)) '.jpg'];
        sn3IM=fullfile(f6,pn2IM);
        exportgraphics(fg2IM,sn3IM,'Resolution',600);
        
        %SEGMENTED_MT
        %save histogram span and pixel count plot image_SEGMENTED MT
        fg2MT=figure('visible','off');
        
        yyaxis left
        laMT=plot(kinMT(:,262),kinMT(:,260));
        hold on
        lbMT=plot(kinMT(:,262),kinMT(:,259));
        lcMT=plot(kinMT(:,262),kinMT(:,258));
        ldMT=plot(kinMT(:,262),kinMT(:,263));
        ylim([0 255]);
        yticks(0:25:255)
        ylabel('Pixel Change Intensity')
        hold off
        
        yyaxis right
        leMT=plot(kinMT(:,262),kinMT(:,261)*100);
        hold on
        lfMT=plot(ones(1,100)*tsupply,linspace(0,100,100),'--');
        hold off
        ylabel('S change (%)')
        ylim([0 100])
        yticks(0:10:100)
        
        legend([laMT,lbMT,lcMT,ldMT,leMT,lfMT],'lth','Max','Min','Mid','Sc%','ts')
        legend('Location','northoutside','Orientation','horizontal')
        
        xlabel('Time (s)')
        xlim([0 dw]);
        xticks(0:dw/6:dw)
        
        pn2MT=[r.name(1:(length(r.name)-4)) '.jpg'];
        sn3MT=fullfile(f9,pn2MT);
        exportgraphics(fg2MT,sn3MT,'Resolution',600);
        
        %close figures
        close(fg);
        close(fgIM);
        close(fgMT);
        close(fg2);
        close(fg2IM);
        close(fg2MT);
        
        %update waitbar
        lwb=lwb+1;
        if lwb==1; eta=1.2*toc; end
        waitbar(lwb/lwbt,wb,sprintf('Analyzing ROI Pixels\n (ETA: %.0f min)',(lwbt-lwb)*eta/60));
    end
end

close(wb);
%% Additional Functions

function [sROI] = ROIread(cstrFilenames)
% ReadImageJROI - FUNCTION Read an ImageJ ROI into a matlab structure
%
% Usage: [sROI] = ReadImageJROI(strFilename)
%        [cvsROIs] = ReadImageJROI(cstrFilenames)
%        [cvsROIs] = ReadImageJROI(strROIArchiveFilename)
%
% This function reads the ImageJ binary ROI file format.
%
% 'strFilename' is the full path to a '.roi' file.  A list of ROI files can be
% passed as a cell array of filenames, in 'cstrFilenames'.  An ImageJ ROI
% archive can be access by providing a '.zip' filename in
% 'strROIArchiveFilename'.  Single ROIs are returned as matlab structures, with
% variable fields depending on the ROI type.  Multiple ROIs are returned as a
% cell array of ROI structures.
%
% The field '.strName' is guaranteed to exist, and contains the ROI name (the
% filename minus '.roi', or the name set for the ROI).
%
% The field '.strType' is guaranteed to exist, and defines the ROI type:
% {'Rectangle', 'Oval', Line', 'Polygon', 'Freehand', 'Traced', 'PolyLine',
% 'FreeLine', 'Angle', 'Point', 'NoROI'}.
%
% The field '.vnRectBounds' is guaranteed to exist, and defines the rectangular
% bounds of the ROI: ['nTop', 'nLeft', 'nBottom', 'nRight'].
%
% The field '.nVersion' is guaranteed to exist, and defines the version number
% of the ROI format.
%
% The field '.vnPosition' is guaranteed to exist. If the information is
% defined within the ROI, this field will be a three-element vector
% [nCPosition nZPosition nTPosition].
%
% ROI types:
%  Rectangle:
%     .strType = 'Rectangle';
%     .nArcSize         - The arc size of the rectangle's rounded corners
%
%      For a composite, 'shape' ROI:
%     .strSubtype = 'Shape';
%     .vfShapeSegments  - A long, complicated vector of complicated shape
%                          segments.  This vector is in the format passed to the
%                          ImageJ ShapeROI constructor.  I won't decode this for
%                          you! :(
%
%  Oval:
%     .strType = 'Oval';
%
%  Line:
%     .strType = 'Line';
%  	.vnLinePoints     - The end points of the line ['nX1', 'nY1', 'nX2', 'nY2']
%
%     With arrow:
%     .strSubtype = 'Arrow';
%     .bDoubleHeaded    - Does the line have two arrowheads?
%     .bOutlined        - Is the arrow outlined?
%     .nArrowStyle      - The ImageJ style of the arrow (unknown interpretation)
%     .nArrowHeadSize   - The size of the arrowhead (unknown units)
%
%  Polygon:
%     .strType = 'Polygon';
%     .mnCoordinates    - An [Nx2] matrix, specifying the coordinates of
%                          the polygon vertices.  Each row is [nX nY].
%
%  Freehand:
%     .strType = 'Freehand';
%     .mnCoordinates    - An [Nx2] matrix, specifying the coordinates of
%                          the polygon vertices.  Each row is [nX nY].
%
%     Ellipse subtype:
%     .strSubtype = 'Ellipse';
%     .vfEllipsePoints  - A vector containing the ellipse control points:
%                          [fX1 fY1 fX2 fY2].
%     .fAspectRatio     - The aspect ratio of the ellipse.
%
%  Traced:
%     .strType = 'Traced';
%     .mnCoordinates    - An [Nx2] matrix, specifying the coordinates of
%                          the line vertices.  Each row is [nX nY].
%
%  PolyLine:
%     .strType = 'PolyLine';
%     .mnCoordinates    - An [Nx2] matrix, specifying the coordinates of
%                          the line vertices.  Each row is [nX nY].
%
%  FreeLine:
%     .strType = 'FreeLine';
%     .mnCoordinates    - An [Nx2] matrix, specifying the coordinates of
%                          the line vertices.  Each row is [nX nY].
%
%  Angle:
%     .strType = 'Angle';
%     .mnCoordinates    - An [Nx2] matrix, specifying the coordinates of
%                          the angle vertices.  Each row is [nX nY].
%
%  Point:
%     .strType = 'Point';
%     .mfCoordinates    - An [Nx2] matrix, specifying the coordinates of
%                          the points.  Each row is [fX fY].
%     .vnCounters       - An [Nx1] vector, specifying which counter is
%                          associated with each point. May be empty.
%     .vnSlices         - An [Nx1] vector, specifying on which plane each
%                          point is on. These are specified as linear
%                          indices, with the hyperstack arranged as
%                          [Channels Z-slices T-slices]. If empty, then all
%                          points are placed on the first slice.
%
%  NoROI:
%     .strType = 'NoROI';
%
% Additionally, ROIs from later versions (.nVersion >= 218) may have the
% following fields:
%
%     .nStrokeWidth     - The width of the line stroke
%     .nStrokeColor     - The encoded color of the stroke (ImageJ color format)
%     .nFillColor       - The encoded fill color for the ROI (ImageJ color
%                          format)
%
% If the ROI contains text:
%     .strSubtype = 'Text';
%     .nFontSize        - The desired font size
%     .nFontStyle       - The style of the font (unknown format)
%     .strFontName      - The name of the font to render the text with
%     .strText          - A string containing the text
% Author: Dylan Muir <dylan.muir@unibas.ch>
% Created: 9th August, 2011
%
% 20170118 Added code to read slice position for point ROIs
% 20141020 Added code to read 'header 2' fields; thanks to Luca Nocetti
% 20140602 Bug report contributed by Samuel Barnes and Yousef Mazaheri
% 20110810 Bug report contributed by Jean-Yves Tinevez
% 20110829 Bug fix contributed by Benjamin Ricca <ricca@berkeley.edu>
% 20120622 Order of ROIs in a ROI set is now preserved
% 20120703 Different way of reading zip file contents guarantees that ROI order
%           is preserved
%
% Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016 Dylan Muir <dylan.muir@unibas.ch>
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% -- Constants
bOpt_SubPixelResolution = 128;
% -- Check arguments
if (nargin < 1)
    disp('*** ReadImageJROI: Incorrect usage');
    help ReadImageJROI;
    return;
end
% -- Check for a cell array of ROI filenames
if (iscell(cstrFilenames))
    % - Read each ROI in turn
    cvsROI = cellfun(@ReadImageJROI, CellFlatten(cstrFilenames), 'UniformOutput', false);
    
    % - Return all ROIs
    sROI = cvsROI;
    return;
    
else
    % - This is not a cell string
    strFilename = cstrFilenames;
    clear cstrFilenames;
end
% -- Check for a zip file
[nul, nul, strExt] = fileparts(strFilename); %#ok<ASGLU>
if (isequal(lower(strExt), '.zip'))
    % - get zp file contents
    cstrFilenames_short = listzipcontents_rois(strFilename);
    
    % - Unzip the file into a temporary directory
    strROIDir = tempname;
    unzip(strFilename, strROIDir);
    
    for (nFileIndex = 1:length(cstrFilenames_short))
        cstrFilenames{1, nFileIndex} = [strROIDir '/' char(cstrFilenames_short(nFileIndex, 1))];
    end
    
    % - Build ROIs for each file
    cvsROIs = ReadImageJROI(cstrFilenames);
    
    % - Clean up temporary directory
    rmdir(strROIDir, 's');
    
    % - Return ROIs
    sROI = cvsROIs;
    return;
end
% -- Read ROI
% -- Check file and open
if (~exist(strFilename, 'file'))
    error('ReadImageJROI:FileNotFound', ...
        '*** ReadImageJROI: The file [%s] was not found.', strFilename);
end
fidROI = fopen(strFilename, 'r', 'ieee-be');
% -- Check file magic code
strMagic = fread(fidROI, [1 4], '*char');
if (~isequal(strMagic, 'Iout'))
    error('ReadImageJROI:FormatError', ...
        '*** ReadImageJROI: The file was not an ImageJ ROI format.');
end
% -- Read version
sROI.nVersion = fread(fidROI, 1, 'int16');
% -- Read ROI type
nTypeID = fread(fidROI, 1, 'uint8');
fseek(fidROI, 1, 'cof'); % Skip a byte
% -- Read rectangular bounds
sROI.vnRectBounds = fread(fidROI, [1 4], 'int16');
% -- Read number of coordinates
nNumCoords = fread(fidROI, 1, 'uint16');
% -- Read the rest of the header
vfLinePoints = fread(fidROI, 4, 'float32');
nStrokeWidth = fread(fidROI, 1, 'int16');
nShapeROISize = fread(fidROI, 1, 'uint32');
nStrokeColor = fread(fidROI, 1, 'uint32');
nFillColor = fread(fidROI, 1, 'uint32');
nROISubtype = fread(fidROI, 1, 'int16');
nOptions = fread(fidROI, 1, 'int16');
nArrowStyle = fread(fidROI, 1, 'uint8');
nArrowHeadSize = fread(fidROI, 1, 'uint8');
nRoundedRectArcSize = fread(fidROI, 1, 'int16');
sROI.nPosition = fread(fidROI, 1, 'uint32');
% -- Read the 'header 2' fields
nHeader2Offset = fread(fidROI, 1, 'uint32');
if (nHeader2Offset > 0) && ~fseek(fidROI, nHeader2Offset+32+4, 'bof')
    % - Seek to start of header 2
    fseek(fidROI, nHeader2Offset+4, 'bof');
    
    % - Read fields
    sROI.vnPosition = fread(fidROI, 3, 'uint32')';
    vnNameParams = fread(fidROI, 2, 'uint32')';
    nOverlayLabelColor = fread(fidROI, 1, 'uint32'); %#ok<NASGU>
    nOverlayFontSize = fread(fidROI, 1, 'int16'); %#ok<NASGU>
    fseek(fidROI, 1, 'cof');   % Skip a byte
    nOpacity = fread(fidROI, 1, 'uint8'); %#ok<NASGU>
    nImageSize = fread(fidROI, 1, 'uint32'); %#ok<NASGU>
    fStrokeWidth = fread(fidROI, 1, 'float32'); %#ok<NASGU>
    vnROIPropertiesParams = fread(fidROI, 2, 'uint32')'; %#ok<NASGU>
    nCountersOffset = fread(fidROI, 1, 'uint32');
    
else
    sROI.vnPosition = [];
    vnNameParams = [0 0];
    nOverlayLabelColor = []; %#ok<NASGU>
    nOverlayFontSize = []; %#ok<NASGU>
    nOpacity = []; %#ok<NASGU>
    nImageSize = []; %#ok<NASGU>
    fStrokeWidth = []; %#ok<NASGU>
    vnROIPropertiesParams = [0 0]; %#ok<NASGU>
    nCountersOffset = 0;
end
% -- Set ROI name
if (isempty(vnNameParams) || any(vnNameParams == 0) || fseek(fidROI, sum(vnNameParams), 'bof'))
    [nul, sROI.strName] = fileparts(strFilename); %#ok<ASGLU>
else
    % - Try to read ROI name from header
    fseek(fidROI, vnNameParams(1), 'bof');
    sROI.strName = fread(fidROI, vnNameParams(2), 'int16=>char')';
end
% - Seek to get aspect ratio
fseek(fidROI, 52, 'bof');
fAspectRatio = fread(fidROI, 1, 'float32');
% - Seek to after header
fseek(fidROI, 64, 'bof');
% -- Build ROI
switch nTypeID
    case 1
        % - Rectangle
        sROI.strType = 'Rectangle';
        sROI.nArcSize = nRoundedRectArcSize;
        
        if (nShapeROISize > 0)
            % - This is a composite shape ROI
            sROI.strSubtype = 'Shape';
            if (nTypeID ~= 1)
                error('ReadImageJROI:FormatError', ...
                    '*** ReadImageJROI: A composite ROI must be a Rectangle type.');
            end
            
            % - Read shapes
            sROI.vfShapes = fread(fidROI, nShapeROISize, 'float32');
        end
        
        
    case 2
        % - Oval
        sROI.strType = 'Oval';
        
    case 3
        % - Line
        sROI.strType = 'Line';
        sROI.vnLinePoints = round(vfLinePoints);
        
        if (nROISubtype == 2)
            % - This is an arrow line
            sROI.strSubtype = 'Arrow';
            sROI.bDoubleHeaded = nOptions & 2;
            sROI.bOutlined = nOptions & 4;
            sROI.nArrowStyle = nArrowStyle;
            sROI.nArrowHeadSize = nArrowHeadSize;
        end
        
        
    case 0
        % - Polygon
        sROI.strType = 'Polygon';
        sROI.mnCoordinates = read_coordinates;
        
    case 7
        % - Freehand
        sROI.strType = 'Freehand';
        sROI.mnCoordinates = read_coordinates;
        
        if (nROISubtype == 3)
            % - This is an ellipse
            sROI.strSubtype = 'Ellipse';
            sROI.vfEllipsePoints = vfLinePoints;
            sROI.fAspectRatio = fAspectRatio;
        end
        
    case 8
        % - Traced
        sROI.strType = 'Traced';
        sROI.mnCoordinates = read_coordinates;
        
    case 5
        % - PolyLine
        sROI.strType = 'PolyLine';
        sROI.mnCoordinates = read_coordinates;
        
    case 4
        % - FreeLine
        sROI.strType = 'FreeLine';
        sROI.mnCoordinates = read_coordinates;
        
    case 9
        % - Angle
        sROI.strType = 'Angle';
        sROI.mnCoordinates = read_coordinates;
        
    case 10
        % - Point
        sROI.strType = 'Point';
        [sROI.mfCoordinates, vnCounters] = read_coordinates;
        
        % - Set counters and [C Z T] positions
        if (isempty(vnCounters))
            sROI.vnCounters = zeros(nNumCoords, 1);
            sROI.vnSlices = ones(nNumCoords, 1);
        else
            sROI.vnCounters = bitand(vnCounters, 255);
            sROI.vnSlices = bitshift(vnCounters, -8, 'uint32');
        end
        
    case 6
        sROI.strType = 'NoROI';
        
    otherwise
        error('ReadImageJROI:FormatError', ...
            '--- ReadImageJROI: The ROI file contains an unknown ROI type.');
end
% -- Handle version >= 218
if (sROI.nVersion >= 218)
    sROI.nStrokeWidth = nStrokeWidth;
    sROI.nStrokeColor = nStrokeColor;
    sROI.nFillColor = nFillColor;
    sROI.bSplineFit = nOptions & 1;
    
    if (nROISubtype == 1)
        % - This is a text ROI
        sROI.strSubtype = 'Text';
        
        % - Seek to after header
        fseek(fidROI, 64, 'bof');
        
        sROI.nFontSize = fread(fidROI, 1, 'uint32');
        sROI.nFontStyle = fread(fidROI, 1, 'uint32');
        nNameLength = fread(fidROI, 1, 'uint32');
        nTextLength = fread(fidROI, 1, 'uint32');
        
        % - Read font name
        sROI.strFontName = fread(fidROI, nNameLength, 'uint16=>char');
        
        % - Read text
        sROI.strText = fread(fidROI, nTextLength, 'uint16=>char');
    end
end
% - Close the file
fclose(fidROI);
% --- END of ReadImageJROI FUNCTION ---
    function [mnCoordinates, vnCounters] = read_coordinates
        
        % - Check for sub-pixel resolution
        if bitand(nOptions, bOpt_SubPixelResolution)
            fseek(fidROI, 64 + 4*nNumCoords, 'bof');
            
            % - Read X and Y coordinates
            vnX = fread(fidROI, [nNumCoords 1], 'single');
            vnY = fread(fidROI, [nNumCoords 1], 'single');
            
        else
            % - Read X and Y coords
            vnX = fread(fidROI, [nNumCoords 1], 'int16');
            vnY = fread(fidROI, [nNumCoords 1], 'int16');
            
            % - Trim at zero
            vnX(vnX < 0) = 0;
            vnY(vnY < 0) = 0;
            
            % - Offset by top left ROI bound
            vnX = vnX + sROI.vnRectBounds(2);
            vnY = vnY + sROI.vnRectBounds(1);
        end
        
        mnCoordinates = [vnX vnY];
        
        % - Read counters, if present
        if (nCountersOffset ~= 0)
            fseek(fidROI, nCountersOffset, 'bof');
            vnCounters = fread(fidROI, [nNumCoords 1], 'uint32');
        else
            vnCounters = [];
        end
    end
    function [filelist] = listzipcontents_rois(zipFilename)
        
        % listzipcontents_rois - FUNCTION Read the file names in a zip file
        %
        % Usage: [filelist] = listzipcontents_rois(zipFilename)
        
        % - Import java libraries
        import java.util.zip.*;
        import java.io.*;
        
        % - Read file list via JAVA object
        filelist={};
        in = ZipInputStream(FileInputStream(zipFilename));
        entry = in.getNextEntry();
        
        % - Filter ROI files
        while (entry~=0)
            name = entry.getName;
            if (name.endsWith('.roi')) && (~name.startsWith('__MACOSX'))
                filelist = cat(1,filelist,char(name));
            end
            entry = in.getNextEntry();
        end
        
        % - Close zip file
        in.close();
    end
    function [cellArray] = CellFlatten(varargin)
        
        % CellFlatten - FUNCTION Convert a list of items to a single level cell array
        %
        % Usage: [cellArray] = CellFlatten(arg1, arg2, ...)
        %
        % CellFlatten will convert a list of arguments into a single-level cell array.
        % If any argument is already a cell array, each cell will be concatenated to
        % 'cellArray' in a list.  The result of this function is a single-dimensioned
        % cell array containing a cell for each individual item passed to CellFlatten.
        % The order of cell elements in the argument list is guaranteed to be
        % preserved.
        %
        % This function is useful when dealing with variable-length argument lists,
        % each item of which can also be a cell array of items.
        
        % Author: Dylan Muir <dylan@ini.phys.ethz.ch>
        % Created: 14th May, 2004
        
        % -- Check arguments
        
        if (nargin == 0)
            disp('*** CellFlatten: Incorrect usage');
            help CellFlatten;
            return;
        end
        
        % -- Convert arguments
        
        % - Which elements contain cell subarrays?
        vbIsCellSubarray = cellfun(@iscell, varargin, 'UniformOutput', true);
        
        % - Skip if no cell subarrays
        if ~any(vbIsCellSubarray)
            cellArray = varargin;
            return;
        end
        
        % - Recursively flatten subarrays
        varargin(vbIsCellSubarray) = cellfun(@(c)CellFlatten(c{:}), varargin(vbIsCellSubarray), 'UniformOutput', false);
        
        % - Count the total number of arguments
        vnArgSizes(nargin) = nan;
        vnArgSizes(~vbIsCellSubarray) = 1;
        vnArgSizes(vbIsCellSubarray) = cellfun(@numel, varargin(vbIsCellSubarray), 'UniformOutput', true);
        vnArgEnds = cumsum(vnArgSizes);
        vnArgStarts = [1 vnArgEnds(1:end-1)+1];
        nNumArgs = vnArgEnds(end);
        
        % - Preallocate return array
        cellArray = cell(1, nNumArgs);
        
        % - Deal out non-cell subarray arguments
        cellArray(vnArgEnds(~vbIsCellSubarray)) = varargin(~vbIsCellSubarray);
        
        % - Deal out arguments into return array
        for nIndexArg = find(vbIsCellSubarray)
            cellArray(vnArgStarts(nIndexArg):vnArgEnds(nIndexArg)) = varargin{nIndexArg};
        end
        
        % --- END of CellFlatten.m ---
    end
end
% --- END of ReadImageJROI.m ---

%% color identifier

function pC=argb2rgb(a)
%outputs RGB values corresponding to input ARGB decimal
%RGB values range from 0 to 1

h=dec2hex(a);
ct=0; pC=zeros(3,1);
for i=3:2:8
    ct=ct+1;
    pC(ct)=hex2dec(h(i:i+1))/255;
end
end

%% ROI mask
function immask = ROImask(im,pxc)
%this functiong generates the mask for isolating pixels on the
%intermetallics defined by the ROIs
%im is the image array
%roif is the roi

imm1=im*0;

for j=1:length(pxc)
    imm1(pxc(j,1),pxc(j,2))=255;
end

imm1=imbinarize(imm1);
immask=uint8(imfill(imm1,'holes'));
end
%% PXC complete
function pxc=pxconnectf(pxc)
%these functions connects the pixels given in the pxc and completes
%the boundary of the IM
lp=length(pxc);
pxc(lp+1,:)=pxc(1,:);
pxn=zeros(2000,2);
pxn(1,:)=pxc(1,:);
co=1;
pxc=round(pxc); %to deal with fractional coordinates sometimes generated by imagej/fiji
for cc=1:lp
    if pxc(cc,1)==pxc(cc+1,1) %same column coord
        dist=pxc(cc+1,2)-pxc(cc,2); %vertical distance
        if dist>0; diff=1; else; diff=-1;end
        for dd=1:(abs(dist)-1)
            pxn(co+dd,:)=[pxc(cc,1) pxc(cc,2)+diff*dd]; %identify coords in between
        end
        co=co+abs(dist);
        pxn(co,:)=pxc(cc+1,:);
    elseif pxc(cc,2)==pxc(cc+1,2) %same row coord
        dist=pxc(cc+1,1)-pxc(cc,1); %horizontal distance
        if dist>0; diff=1; else; diff=-1;end
        for dd=1:(abs(dist)-1)
            pxn(co+dd,:)=[pxc(cc,1)+diff*dd  pxc(cc,2)]; %identify coords in between
        end
        co=co+abs(dist);
        pxn(co,:)=pxc(cc+1,:);
    end
end
pxn((co+1):end,:)=[];
pxc=pxn;
end

%% median GL search
function midGL=midGLsearch(fd)
%cumulative distribution
cmfrqdist=fd*0;
cmfrqdist(1)=fd(1);
if cmfrqdist(1)>=0.5
    midGL=0;
else
    for binv=2:length(fd)
        cmfrqdist(binv)=cmfrqdist(binv-1)+fd(binv);
        if cmfrqdist(binv)>=0.5
            midGL=binv-1;
            break
        end
    end
end
end